############################
# **** IMPORT SECTION **** #
############################

import hal
from qtvcp.core import Status, Action
from qtvcp.widgets.camview_widget import CamView as CAM
# Set up logging
from qtvcp import logger
from PyQt5.QtWidgets import QPushButton as PBUTTON
from PyQt5.QtWidgets import QLabel as LABEL
from PyQt5 import QtCore


###########################################
# **** instantiate libraries section **** #
###########################################

STATUS = Status()
ACTION = Action()
LOG = logger.getLogger(__name__)
# Set the log level for this module
#LOG.setLevel(logger.INFO) # One of DEBUG, INFO, WARNING, ERROR, CRITICAL

###################################
# **** HANDLER CLASS SECTION **** #
###################################

class HandlerClass:

    ########################
    # **** INITIALIZE **** #
    ########################
    # widgets allows access to  widgets from the qtvcp files
    # at this point the widgets and hal pins are not instantiated
    def __init__(self, halcomp,widgets,paths):
        self.hal = halcomp
        self.w = widgets
        self.PATHS = paths
        self.degree = u"\N{DEGREE SIGN}".encode('utf-8')

    ##########################################
    # Special Functions called from QTVCP
    ##########################################

    # at this point:
    # the widgets are instantiated.
    # the HAL pins are built but HAL is not set ready
    def initialized__(self):
        self.pin_ = self.hal.newpin('cam-rotation',hal.HAL_FLOAT, hal.HAL_OUT)
        if self.w.PREFS_:
            print 'Using preference file:',self.w.PREFS_.fn
        self.w.actionbutton.setText('TO Origin')
        self.w.dro_label.metric_template = 'X:%0.3f'
        self.w.dro_label.imperial_template = 'X:%0.4f'
        self.w.dro_label_2.metric_template = 'Y:%0.3f'
        self.w.dro_label_2.imperial_template = 'Y:%0.4f'
        self.w.dro_label_3.setHidden(True)
        self.w.camview.rotationIncrement = 1.00
        self.w.rotate_degrees = PBUTTON()
        self.w.rotate_degrees.resize(self.w.rotate_degrees.sizeHint())
        self.w.rotate_degrees.clicked.connect(self.rotation_changed)
        self.w.horizontalLayout.insertWidget(2,self.w.rotate_degrees)
        self.w.rotate_degrees.show()
        self.w.rotate_degrees.setText('Rot {:0.2f}{}' \
          .format(self.w.camview.rotationIncrement, self.degree))

    def class_patch__(self):
        self.super_nectFrameSlot = CAM.nextFrameSlot
        CAM.nextFrameSlot = self.nextFrame
        self.old_mouseDoubleClickEvent = CAM.mouseDoubleClickEvent
        CAM.mouseDoubleClickEvent = self.doubleClick


    ########################
    # callbacks from STATUS #
    ########################

    #######################
    # callbacks from form #
    #######################

    #####################
    # general functions #
    #####################
    def nextFrame(self, w):
        self.super_nectFrameSlot(self.w.camview, None)
        self.pin_.set(360 - self.w.camview.rotation)

    def doubleClick(self, event):
        if event.button() & QtCore.Qt.LeftButton:
            self.w.camview.scale = 1
        elif event.button() & QtCore.Qt.RightButton:
            self.w.camview.rotation = 0
        elif event.button() & QtCore.Qt.MiddleButton:
            self.w.camview.diameter = 20

    def rotation_changed(self,w):
        if self.w.camview.rotationIncrement == 1.00:
            self.w.camview.rotationIncrement = 0.1
        elif self.w.camview.rotationIncrement == 0.10:
            self.w.camview.rotationIncrement = 0.01
        else:
            self.w.camview.rotationIncrement = 1.00
        self.w.rotate_degrees.setText('Rot {:0.2f}{}'\
          .format(self.w.camview.rotationIncrement, self.degree))

    #####################
    # KEY BINDING CALLS #
    #####################

    ###########################
    # **** closing event **** #
    ###########################

    ##############################
    # required class boiler code #
    ##############################

    def __getitem__(self, item):
        return getattr(self, item)
    def __setitem__(self, item, value):
        return setattr(self, item, value)

################################
# required handler boiler code #
################################

def get_handlers(halcomp,widgets,paths):
     return [HandlerClass(halcomp,widgets,paths)]